@@ -21,8 +21,9 @@ __all__ = (
     "BackgroundSystemModel",
     "ExportInventory",
     "InternalNoiseModel",
+    "create_fleet_composition_from_IAM_file",
 )
-__version__ = (1, 5, 6)
+__version__ = (1, 6, 0)
 
 from pathlib import Path
 
@@ -42,3 +43,4 @@ from .internal_noise import InternalNoiseModel
 from .inventory import InventoryCalculation
 from .model import CarModel
 from .noise_emissions import NoiseEmissionsModel
+from .utils import create_fleet_composition_from_IAM_file
