@@ -1,6 +1,7 @@
-from klausen import NamedParameters
-from pathlib import Path
 import json
+from pathlib import Path
+
+from klausen import NamedParameters
 
 DEFAULT = Path(__file__, "..").resolve() / "data" / "default_parameters.json"
 EXTRA = Path(__file__, "..").resolve() / "data" / "extra_parameters.json"
@@ -30,7 +31,7 @@ class CarInputParameters(NamedParameters):
     :vartype powertrains: list
     :ivar parameters: List of string items e.g., ['Benzene', 'CH4', 'CNG tank mass intercept',...]
     :vartype parameters: list
-    :ivar years: List of integers e.g., [2017, 2040]
+    :ivar years: List of integers e.g., [2000, 2010, 2020, 2040]
     :vartype years: list
     :ivar metadata: Dictionary for metadata.
     :vartype metadata: dict
@@ -73,9 +74,7 @@ class CarInputParameters(NamedParameters):
         )
 
         # keep a list of input parameters, for sensitivity purpose
-        self.input_parameters = sorted(
-            {o["name"] for o in parameters.values()})
-
+        self.input_parameters = sorted({o["name"] for o in parameters.values()})
 
         self.years = sorted({o["year"] for o in parameters.values()})
         self.add_car_parameters(parameters)
