@@ -1,38 +1,121 @@
+import pickle
+
 import numpy as np
 import xarray as xr
-import pickle
+
 from . import DATA_DIR
 
 
 def _(o):
     """Add a trailing dimension to make input arrays broadcast correctly"""
-    if isinstance(o, (np.ndarray, xarray.DataArray)):
+    if isinstance(o, (np.ndarray, xr.DataArray)):
         return np.expand_dims(o, -1)
     else:
         return o
 
+
 def get_hot_emission_factors():
-    """ Hot emissions factors extracted for trucks from HBEFA 4.1
-        detailed by size, powertrain and EURO class for each substance.
+    """Hot emissions factors extracted for trucks from HBEFA 4.1
+    detailed by size, powertrain and EURO class for each substance.
     """
     fp = DATA_DIR / "hot.pickle"
 
-    with open(fp, 'rb') as f:
+    with open(fp, "rb") as f:
         hot = pickle.load(f)
 
     return hot
 
+
 def get_non_hot_emission_factors():
-    """ Non hot emissions factors (cold start, evaporation, soak emissions) extracted for trucks from HBEFA 4.1
-        detailed by size, powertrain and EURO class for each substance.
+    """Non hot emissions factors (cold start, evaporation, soak emissions) extracted for trucks from HBEFA 4.1
+    detailed by size, powertrain and EURO class for each substance.
     """
     fp = DATA_DIR / "non_hot.pickle"
 
-    with open(fp, 'rb') as f:
+    with open(fp, "rb") as f:
         non_hot = pickle.load(f)
 
     return non_hot
 
+
+def get_mileage_degradation_factor(powertrain_type, euro_class, lifetime_km):
+    """
+    Catalyst degrade overtime, leading to increased emissions
+    of CO, HC and NOX. We apply a correction factor
+    to reflect this.
+    :return:
+    """
+
+    d_corr = {
+        "ICEV-p": {
+            "CO": {
+                1: 1.9,
+                2: 1.6,
+                3: 1.75,
+                4: 1.9,
+                5: 2,
+                6: 1.3,
+                6.1: 1.3,
+                6.2: 1.3,
+                6.3: 1.3,
+            },
+            "HC": {1: 1.59, 2: 1.59, 3: 1.02, 4: 1.02},
+            "NOx": {
+                1: 2.5,
+                2: 2.3,
+                3: 2.9,
+                4: 2,
+                5: 2.5,
+                6: 1.3,
+                6.1: 1.3,
+                6.2: 1.3,
+                6.3: 1.3,
+            },
+        },
+        "ICEV-d": {
+            "CO": {
+                4: 1.3,
+                5: 1.3,
+                6: 1.4,
+                6.1: 1.4,
+                6.2: 1.4,
+                6.3: 1.4,
+            },
+            "NOx": {
+                2: 1.25,
+                3: 1.2,
+                4: 1.06,
+                5: 1.03,
+                6: 1.15,
+                6.1: 1.15,
+                6.2: 1.15,
+                6.3: 1.15,
+            },
+        },
+    }
+
+    shape = list(lifetime_km.shape)
+    shape[-1] = 3
+    corr = np.ones(tuple(shape))
+
+    for p, pt in enumerate(powertrain_type):
+        for e, ec in enumerate(euro_class):
+            for c, co in enumerate(["CO", "HC", "NOx"]):
+                try:
+                    val = d_corr[pt][co][ec]
+                    y_max = 120000 if co == "HC" else 200000
+                    corr[:, p, e, c] = np.clip(
+                        np.interp(lifetime_km[:, p, e, 0] / 2, [0, y_max], [1, val]),
+                        1,
+                        None,
+                    )
+
+                except KeyError:
+                    pass
+
+    return corr.transpose(3, 0, 1, 2)
+
+
 class HotEmissionsModel:
     """
     Calculate hot pollutants emissions based on HBEFA 4.1 data, function of fuel consumption
@@ -86,7 +169,9 @@ class HotEmissionsModel:
         self.hot = get_hot_emission_factors()
         self.non_hot = get_non_hot_emission_factors()
 
-    def get_hot_emissions(self, powertrain_type, euro_class, energy_consumption, yearly_km):
+    def get_hot_emissions(
+        self, powertrain_type, euro_class, lifetime_km, energy_consumption, yearly_km
+    ):
         """
         Calculate hot pollutants emissions given a powertrain type (i.e., diesel, petrol, CNG) and a EURO pollution class, per air sub-compartment
         (i.e., urban, suburban and rural).
@@ -141,9 +226,20 @@ class HotEmissionsModel:
         # the fitting of emissions function of energy consumption is described in the notebook
         # `HBEFA trucks.ipynb` in the folder `dev`.
 
-
         # energy conusmption is given in kj for each second
-        hot = hot_emissions.sel(variable="a").values[:, None, :, :, None, None] * energy_consumption.values
+        # emissions are in grams per MJ
+        hot = (
+            hot_emissions.sel(variable="a").values[:, None, :, :, None, None]
+            * energy_consumption.values
+        )
+        # bit of a manual calibration for N2O and NH3
+        # as they do not correlate with fuel consumption
+        hot[6] *= 0.5
+        hot[7] *= 0.5
+
+        # apply a mileage degradation factor for CO, HC and NOx
+        corr = get_mileage_degradation_factor(powertrain_type, euro_class, lifetime_km)
+        hot[:3] *= corr[:, :, :, :, None, None]
 
         non_hot_emissions = self.non_hot.sel(
             powertrain=powertrain_type,
@@ -162,25 +258,31 @@ class HotEmissionsModel:
             ],
         ).transpose("Component", "powertrain", "euro_class", "type")
 
-        start_per_day = 3
+        start_per_day = 2.3  # source for
 
         # Cold start and soak emissions are defined per start and stop
         # Therefore, we need to normalize per km
         # And add cold start emissions to the first second of the driving cycle
-        hot[..., 0] += ((start_per_day * 365 / yearly_km).values
-              * non_hot_emissions.loc[dict(type="cold start")].values[:, None, ..., None])
+        hot[..., 0] += (start_per_day * 365 / yearly_km).values * non_hot_emissions.loc[
+            dict(type="cold start")
+        ].values[:, None, ..., None]
         # And add soak emissions to the last second of the driving cycle
-        hot[..., -1] += ((start_per_day * 365 / yearly_km).values
-                        * non_hot_emissions.loc[dict(type="soak")].values[:, None, ..., None])
+        hot[..., -1] += (
+            start_per_day * 365 / yearly_km
+        ).values * non_hot_emissions.loc[dict(type="soak")].values[:, None, ..., None]
 
         # Diurnal emissions are defined in g/day
         # And need to be evenly distributed throughout the driving cycle
-        hot += ((365 / yearly_km).values * non_hot_emissions.loc[dict(type="diurnal")].values[:, None, ..., None])[..., None]\
-               / len(self.cycle)
+        hot += (
+            (365 / yearly_km).values
+            * non_hot_emissions.loc[dict(type="diurnal")].values[:, None, ..., None]
+        )[..., None] / len(self.cycle)
 
         # Running losses are in g/km (no conversion needed)
         # And need to be evenly distributed throughout the driving cycle
-        hot += non_hot_emissions.loc[dict(type="running losses")].values[:, None, ..., None, None] / len(self.cycle)
+        hot += non_hot_emissions.loc[dict(type="running losses")].values[
+            :, None, ..., None, None
+        ] / len(self.cycle)
 
         # Add additional species derived from NMHC emissions
         # Toluene, Xylene, Formaldehyde, Acetaldehyde, etc.
@@ -190,12 +292,14 @@ class HotEmissionsModel:
 
         arr = np.zeros(shape)
 
-        arr[:hot.shape[0],
-            :hot.shape[1],
-            :hot.shape[2],
-            :hot.shape[3],
-            :hot.shape[4],
-            :hot.shape[5]] = hot
+        arr[
+            : hot.shape[0],
+            : hot.shape[1],
+            : hot.shape[2],
+            : hot.shape[3],
+            : hot.shape[4],
+            : hot.shape[5],
+        ] = hot
 
         final_emissions = xr.DataArray(
             arr,
@@ -240,152 +344,209 @@ class HotEmissionsModel:
                     "Chromium",
                     "Chromium VI",
                     "Mercury",
-                    "Cadmium"
+                    "Cadmium",
                 ],
                 energy_consumption.coords["size"].values,
                 energy_consumption.powertrain.values,
                 energy_consumption.year.values,
                 range(0, arr.shape[4]),
-                range(0, arr.shape[-1])
+                range(0, arr.shape[-1]),
             ],
             dims=["component", "size", "powertrain", "year", "value", "second"],
         )
 
-        NMHC_ratios_petrol = np.array([0.032,0.007,0.052,0.022,0.016,0.011,0.007,0.073,0.038,
-                        0.001,0.110,0.054,0.023,0.017,0.008,0.002,0.006,0.001,
-                        0.002,0.010])
+        NMHC_ratios_petrol = np.array(
+            [
+                0.032,
+                0.007,
+                0.052,
+                0.022,
+                0.016,
+                0.011,
+                0.007,
+                0.073,
+                0.038,
+                0.001,
+                0.110,
+                0.054,
+                0.023,
+                0.017,
+                0.008,
+                0.002,
+                0.006,
+                0.001,
+                0.002,
+                0.010,
+            ]
+        )
 
+        NMHC_ratios_diesel = np.array(
+            [
+                0.0033,
+                0.0011,
+                0.0011,
+                0.0004,
+                0.0000,
+                0.0065,
+                0.0020,
+                0.1097,
+                0.0360,
+                0.0000,
+                0.0069,
+                0.0061,
+                0.0027,
+                0.1200,
+                0.0647,
+                0.0086,
+                0.0294,
+                0.0120,
+                0.0358,
+                0.0037,
+            ]
+        )
 
-        NMHC_ratios_diesel = np.array([0.0033,0.0011,0.0011,0.0004,0.0000,0.0065,0.0020,
-                                0.1097,0.0360,0.0000,0.0069,0.0061,0.0027,0.1200,
-                                0.0647,0.0086,0.0294,0.0120,0.0358,0.0037])
+        nmvoc_sub_species = [
+            "Ethane",
+            "Propane",
+            "Butane",
+            "Pentane",
+            "Hexane",
+            "Cyclohexane",
+            "Heptane",
+            "Ethene",
+            "Propene",
+            "1-Pentene",
+            "Toluene",
+            "m-Xylene",
+            "o-Xylene",
+            "Formaldehyde",
+            "Acetaldehyde",
+            "Benzaldehyde",
+            "Acetone",
+            "Methyl ethyl ketone",
+            "Acrolein",
+            "Styrene",
+        ]
 
         final_emissions.loc[
             dict(
-                component=["Ethane",
-                            "Propane",
-                            "Butane",
-                            "Pentane",
-                            "Hexane",
-                            "Cyclohexane",
-                            "Heptane",
-                            "Ethene",
-                            "Propene",
-                            "1-Pentene",
-                            "Toluene",
-                            "m-Xylene",
-                            "o-Xylene",
-                            "Formaldehyde",
-                            "Acetaldehyde",
-                            "Benzaldehyde",
-                            "Acetone",
-                            "Methyl ethyl ketone",
-                            "Acrolein",
-                            "Styrene",
-                           ],
-                powertrain=[p for p in final_emissions.powertrain.values if "d" in p]
+                component=nmvoc_sub_species,
+                powertrain=[p for p in final_emissions.powertrain.values if "d" in p],
             )
         ] = (
-                    NMHC_ratios_diesel.reshape((-1, 1, 1, 1, 1, 1)) *
-                    final_emissions.loc[
-                        dict(
-                            component="NMHC",
-                            powertrain=[p for p in final_emissions.powertrain.values if "-d" in p]
-                        )
-                    ].values
-
-            )
-
-        final_emissions.loc[
-            dict(
-                component="NMHC",
-                powertrain=[p for p in final_emissions.powertrain.values if "-d" in p]
-            )
-        ] *= (1 - .45)
-
-        final_emissions.loc[
-            dict(
-                component=["Ethane",
-                            "Propane",
-                            "Butane",
-                            "Pentane",
-                            "Hexane",
-                            "Cyclohexane",
-                            "Heptane",
-                            "Ethene",
-                            "Propene",
-                            "1-Pentene",
-                            "Toluene",
-                            "m-Xylene",
-                            "o-Xylene",
-                            "Formaldehyde",
-                            "Acetaldehyde",
-                            "Benzaldehyde",
-                            "Acetone",
-                            "Methyl ethyl ketone",
-                            "Acrolein",
-                            "Styrene",],
-                powertrain=[p for p in final_emissions.powertrain.values if "-p" in p]
-            )
-        ] = (
-                NMHC_ratios_petrol.reshape((-1, 1, 1, 1, 1, 1)) *
-                final_emissions.loc[
-                    dict(
-                        component="NMHC",
-                        powertrain=[p for p in final_emissions.powertrain.values if "-p" in p]
-                    )
-                ].values
-
+            NMHC_ratios_diesel.reshape((-1, 1, 1, 1, 1, 1))
+            * final_emissions.loc[
+                dict(
+                    component="NMHC",
+                    powertrain=[
+                        p for p in final_emissions.powertrain.values if "-d" in p
+                    ],
+                )
+            ].values
         )
 
         final_emissions.loc[
             dict(
                 component="NMHC",
-                powertrain=[p for p in final_emissions.powertrain.values if "-p" in p]
+                powertrain=[p for p in final_emissions.powertrain.values if "-d" in p],
             )
-        ] *= (1 - .492)
+        ] *= (
+            1 - 0.45
+        )
+
+        final_emissions.loc[
+            dict(
+                component=nmvoc_sub_species,
+                powertrain=[p for p in final_emissions.powertrain.values if "-p" in p],
+            )
+        ] = (
+            NMHC_ratios_petrol.reshape((-1, 1, 1, 1, 1, 1))
+            * final_emissions.loc[
+                dict(
+                    component="NMHC",
+                    powertrain=[
+                        p for p in final_emissions.powertrain.values if "-p" in p
+                    ],
+                )
+            ].values
+        )
+
+        final_emissions.loc[
+            dict(
+                component="NMHC",
+                powertrain=[p for p in final_emissions.powertrain.values if "-p" in p],
+            )
+        ] *= (
+            1 - 0.492
+        )
 
         # Heavy metals emissions are dependent of fuel consumption
-        # given in kg of emission per kj
+        # given in grams of emission per kj
+        heavy_metals = [
+            "PAH, polycyclic aromatic hydrocarbons",
+            "Arsenic",
+            "Selenium",
+            "Zinc",
+            "Copper",
+            "Nickel",
+            "Chromium",
+            "Chromium VI",
+            "Mercury",
+            "Cadmium",
+        ]
         final_emissions.loc[
             dict(
-                component=[
-                    "PAH, polycyclic aromatic hydrocarbons",
-                    "Arsenic",
-                    "Selenium",
-                    "Zinc",
-                    "Copper",
-                    "Nickel",
-                    "Chromium",
-                    "Chromium VI",
-                    "Mercury",
-                    "Cadmium",
-                     ],
-                powertrain=[p for p in final_emissions.powertrain.values if "-p" in p]
+                component=heavy_metals,
+                powertrain=[p for p in final_emissions.powertrain.values if "-p" in p],
             )
-        ] = np.array([4.76E-10, 7.08E-12, 4.72E-12, 5.09E-08, 9.91E-10, 3.07E-10, 3.77E-10,
-                        7.55E-13, 2.05E-10, 2.55E-10]).reshape((-1, 1, 1, 1, 1, 1))\
-                * energy_consumption.sel(powertrain=[p for p in energy_consumption.powertrain.values if "-p" in p]).values
+        ] = (
+            np.array(
+                [
+                    8.19e-10,
+                    7.06e-12,
+                    4.71e-12,
+                    5.08e-08,
+                    9.88e-10,
+                    3.06e-10,
+                    3.76e-10,
+                    7.53e-13,
+                    2.05e-10,
+                    2.54e-10,
+                ]
+            ).reshape((-1, 1, 1, 1, 1, 1))
+            * energy_consumption.sel(
+                powertrain=[
+                    p for p in energy_consumption.powertrain.values if "-p" in p
+                ]
+            ).values
+        )
 
         final_emissions.loc[
             dict(
-                component=[
-                    "PAH, polycyclic aromatic hydrocarbons",
-                    "Arsenic",
-                    "Selenium",
-                    "Zinc",
-                    "Copper",
-                    "Nickel",
-                    "Chromium",
-                    "Chromium VI",
-                    "Mercury",
-                    "Cadmium",
-                ],
-                powertrain=[p for p in final_emissions.powertrain.values if "-d" in p]
+                component=heavy_metals,
+                powertrain=[p for p in final_emissions.powertrain.values if "-d" in p],
             )
-        ] = np.array([1.33E-09, 2.34E-12, 2.34E-12, 4.07E-08, 4.95E-10, 2.06E-10,
-                        7.01E-10, 7.48E-13, 1.24E-10, 2.03E-10]).reshape((-1, 1, 1, 1, 1, 1))\
-            * energy_consumption.sel(powertrain=[p for p in energy_consumption.powertrain.values if "-d" in p]).values
+        ] = (
+            np.array(
+                [
+                    1.32e-09,
+                    2.33e-12,
+                    2.33e-12,
+                    4.05e-08,
+                    4.93e-10,
+                    2.05e-10,
+                    6.98e-10,
+                    1.40e-12,
+                    1.23e-10,
+                    2.02e-10,
+                ]
+            ).reshape((-1, 1, 1, 1, 1, 1))
+            * energy_consumption.sel(
+                powertrain=[
+                    p for p in energy_consumption.powertrain.values if "-d" in p
+                ]
+            ).values
+        )
 
         if self.cycle_name in self.cycle_environment:
             if "urban start" in self.cycle_environment[self.cycle_name]:
@@ -395,7 +556,9 @@ class HotEmissionsModel:
                 urban_start, urban_stop = [0, 0]
 
             if "suburban start" in self.cycle_environment[self.cycle_name]:
-                suburban_start = self.cycle_environment[self.cycle_name]["suburban start"]
+                suburban_start = self.cycle_environment[self.cycle_name][
+                    "suburban start"
+                ]
                 suburban_stop = self.cycle_environment[self.cycle_name]["suburban stop"]
             else:
                 suburban_start, suburban_stop = [0, 0]
@@ -406,16 +569,30 @@ class HotEmissionsModel:
             else:
                 rural_start, rural_stop = [0, 0]
 
-            urban = final_emissions.sel(second=range(urban_start, urban_stop)).sum(axis=-1) / distance
-            suburban = final_emissions.sel(second=range(suburban_start, suburban_stop)).sum(axis=-1) / distance
-            rural = final_emissions.sel(second=range(rural_start, rural_stop)).sum(axis=-1) / distance
-
+            urban = (
+                final_emissions.sel(second=range(urban_start, urban_stop)).sum(axis=-1)
+                / distance
+            )
+            suburban = (
+                final_emissions.sel(second=range(suburban_start, suburban_stop)).sum(
+                    axis=-1
+                )
+                / distance
+            )
+            rural = (
+                final_emissions.sel(second=range(rural_start, rural_stop)).sum(axis=-1)
+                / distance
+            )
 
         else:
-            urban = final_emissions.where(self.cycle <=50).sum(axis=-1) / distance
-            suburban = final_emissions.where((self.cycle >50)&(self.cycle <=80)).sum(axis=-1) / distance
-            rural = final_emissions.where(self.cycle >80).sum(axis=-1) / distance
-
+            urban = final_emissions.where(self.cycle <= 50).sum(axis=-1) / distance
+            suburban = (
+                final_emissions.where((self.cycle > 50) & (self.cycle <= 80)).sum(
+                    axis=-1
+                )
+                / distance
+            )
+            rural = final_emissions.where(self.cycle > 80).sum(axis=-1) / distance
 
         urban *= np.isfinite(urban)
         suburban *= np.isfinite(suburban)
@@ -425,5 +602,4 @@ class HotEmissionsModel:
         suburban = suburban.fillna(0)
         rural = rural.fillna(0)
 
-
         return np.vstack((urban, suburban, rural)).transpose(1, 2, 0, 3, 4) / 1000
